@@ -30,14 +30,22 @@ is.proj <- function(crs) {
 	x@ptr$get_crs("proj4")
 }
 
+.name_from_wkt <- function(wkt) {
+	s = strsplit(wkt, ",")[[1]][1]
+	strsplit(s, "\"")[[1]][[2]]
+}
+
 .name_or_proj4 <- function(x) {
 	if (inherits(x, "SpatVectorProxy")) {
-		v <- vect()
-		v@ptr <- x@ptr$v
-		x <- v
+		ptr <- x@ptr$v
+	} else if (inherits(x, "Rcpp_SpatRaster")) {
+		ptr <- x	
+	} else {
+		ptr <- x@ptr
 	}
-	d <- .srs_describe(x@ptr$get_crs("wkt"))
-	r <- x@ptr$get_crs("proj4")
+	wkt <- ptr$get_crs("wkt")
+	d <- .srs_describe(wkt)
+	r <- ptr$get_crs("proj4")
 	if (!(d$name %in% c(NA, "unknown", "unnamed"))) {
 		if (substr(r, 1, 13) == "+proj=longlat") {
 			r <- paste("lon/lat", d$name)
@@ -48,6 +56,12 @@ is.proj <- function(crs) {
 			r <- paste0(r, " (", d$authority, ":", d$code, ")")
 		}
 	}
+	if (r == "") {
+		rr <- try(.name_from_wkt(wkt), silent=TRUE)
+		if (!inherits(rr, "try-error")) {
+			r <- rr
+		}
+	}
 	r
 }
 
@@ -89,6 +103,13 @@ is.proj <- function(crs) {
 }
 
 
+setMethod("crs", signature("character"),
+	function(x, proj=FALSE, describe=FALSE, parse=FALSE) {
+		x <- rast(crs=x)
+		.get_CRS(x, proj=proj, describe=describe, parse=parse)
+	}
+)
+
 setMethod("crs", signature("SpatRaster"),
 	function(x, proj=FALSE, describe=FALSE, parse=FALSE) {
 		.get_CRS(x, proj=proj, describe=describe, parse=parse)
@@ -122,6 +143,12 @@ setMethod("crs", signature("SpatRasterDataset"),
 		x <- y
 	} else if (is.character(x)) {
 		x <- x[1]
+		lowx <- tolower(x)
+		if (lowx == "local") {
+			x = 'LOCAL_CS["Cartesian (Meter)", LOCAL_DATUM["Local Datum",0], UNIT["Meter",1.0], AXIS["X",EAST], AXIS["Y",NORTH]]'
+		} else if (lowx == "lonlat") {
+			x <- "+proj=longlat"
+		}
 	} else {
 		error("crs", "I do not know what to do with this argument (expected a character string)")
 	}
@@ -250,3 +277,9 @@ setMethod("is.lonlat", signature("SpatVector"),
 	}
 )
 
+setMethod("is.lonlat", signature("character"),
+	function(x, perhaps=FALSE, warn=TRUE) {
+		x <- rast(crs=x)
+		is.lonlat(x, perhaps=perhaps, warn=warn)
+	}
+)
