@@ -1,9 +1,11 @@
 #include <Rcpp.h>
 //#include "spatRaster.h"
 #include "spatRasterMultiple.h"
+#include "spatGraph.h"
 #include <memory> //std::addressof
 #include "NA.h"
 #include "spatTime.h"
+//#include "spatVector2.h"
 
 //static void SpatRaster_finalizer( SpatRaster* ptr ){
 //}
@@ -179,6 +181,8 @@ RCPP_EXPOSED_CLASS(SpatRasterStack)
 RCPP_EXPOSED_CLASS(SpatVector)
 RCPP_EXPOSED_CLASS(SpatVectorProxy)
 RCPP_EXPOSED_CLASS(SpatVectorCollection)
+RCPP_EXPOSED_CLASS(SpatGraph)
+//RCPP_EXPOSED_CLASS(SpatVector2)
 
 RCPP_MODULE(spat){
 
@@ -198,9 +202,16 @@ RCPP_MODULE(spat){
 		.field("labels", &SpatFactor::labels)
 	;
 
+
     class_<SpatSRS>("SpatSRS")
-		.method("is_lonlat", &SpatSRS::is_lonlat, "")
-		.method("to_meter", &SpatSRS::to_meter, "to_meter")
+		.constructor()
+		.method("set", &SpatSRS::set)
+		.method("is_lonlat", &SpatSRS::is_lonlat)
+		.method("to_meter", &SpatSRS::to_meter)
+	;
+
+    class_<SpatGraph>("SpatGraph")
+		.constructor()
 	;
 
     class_<SpatExtent>("SpatExtent")
@@ -362,6 +373,14 @@ RCPP_MODULE(spat){
 		.constructor()
 		.constructor<SpatExtent, std::string>()
 		.constructor<std::vector<std::string>>()
+	;
+
+
+    class_<SpatVector>("SpatVector")
+		.constructor()
+		.constructor<SpatExtent, std::string>()
+		.constructor<std::vector<std::string>>()
+
 		.method("deepcopy", &SpatVector::deepCopy)
 		.method("wkt", &SpatVector::wkt)
 		.method("wkb", &SpatVector::wkb)
@@ -371,7 +390,8 @@ RCPP_MODULE(spat){
 		.method("boundary", &SpatVector::boundary)
 		.method("polygonize", &SpatVector::polygonize)
 		.method("normalize", &SpatVector::normalize)
-		.method("normalize_dateline", &SpatVector::normalize_dateline)
+		.method("normalize_longitude", &SpatVector::normalize_longitude)
+		.method("rotate_longitude", &SpatVector::rotate_longitude)
 		.method("line_merge", &SpatVector::line_merge)
 		.method("simplify", &SpatVector::simplify)
 		.method("thin", &SpatVector::thin)
@@ -403,6 +423,7 @@ RCPP_MODULE(spat){
 		.method("linesList", &SpatVector::linesList)
 		.method("polygonsList", &SpatVector::polygonsList)
 		.method("linesNA", &SpatVector::linesNA)
+		.method("linedistlonlat", &SpatVector::linedistLonLat)
 
 		.method("add_column_empty", (void (SpatVector::*)(unsigned dtype, std::string name))( &SpatVector::add_column))
 		.method("add_column_double", (bool (SpatVector::*)(std::vector<double>, std::string name))( &SpatVector::add_column))
@@ -415,19 +436,19 @@ RCPP_MODULE(spat){
 		.method("remove_column", (bool (SpatVector::*)(int i))( &SpatVector::remove_column))
 		.method("remove_df", &SpatVector::remove_df)
 		.method("set_df", &SpatVector::set_df)
-		.method("get_datatypes", &SpatVector::get_datatypes, "")
+		.method("get_datatypes", &SpatVector::get_datatypes)
 
-		.method("set_holes", &SpatVector::set_holes, "set_holes")
-		.method("get_holes", &SpatVector::get_holes, "get_holes")
-		.method("remove_holes", &SpatVector::remove_holes, "remove holes")
-		.method("append", &SpatVector::append, "append")
-		.method("cbind", &SpatVector::cbind, "cbind")
-		.method("rbind", &SpatVector::append, "rbind")
+		.method("set_holes", &SpatVector::set_holes)
+		.method("get_holes", &SpatVector::get_holes)
+		.method("remove_holes", &SpatVector::remove_holes)
+		.method("append", &SpatVector::append)
+		.method("cbind", &SpatVector::cbind)
+		.method("rbind", &SpatVector::append)
 
-		.method("area", &SpatVector::area, "area")
-		.method("as_lines", &SpatVector::as_lines, "as_lines")
-		.method("as_points", &SpatVector::as_points, "as_points")
-		.method("couldBeLonLat", &SpatVector::could_be_lonlat, "couldBeLonLat")
+		.method("area", &SpatVector::area)
+		.method("as_lines", &SpatVector::as_lines)
+		.method("as_points", &SpatVector::as_points)
+		.method("couldBeLonLat", &SpatVector::could_be_lonlat)
 		.method("get_crs", &SpatVector::getSRS)
 
 		.method("set_crs", (bool (SpatVector::*)(std::string crs))( &SpatVector::setSRS))
@@ -436,39 +457,42 @@ RCPP_MODULE(spat){
 		.method("distance_self", (std::vector<double> (SpatVector::*)(bool, std::string))( &SpatVector::distance))
 		.method("distance_other", (std::vector<double> (SpatVector::*)(SpatVector, bool, std::string))( &SpatVector::distance))
 
+		.method("point_distance", &SpatVector::pointdistance)
+
 		.method("geosdist_self", (std::vector<double> (SpatVector::*)(bool, std::string))( &SpatVector::geos_distance))
 		.method("geosdist_other", (std::vector<double> (SpatVector::*)(SpatVector, bool, std::string))( &SpatVector::geos_distance))
 
-		.method("extent", &SpatVector::getExtent, "extent")
-		.method("getDF", &getVectorAttributes, "get attributes")
-		.method("getGeometryWKT", &SpatVector::getGeometryWKT, "getGeometryWKT")
-		.method("isLonLat", &SpatVector::is_lonlat, "is lonlat")
-		.method("length", &SpatVector::length, "length")
+		.method("extent", &SpatVector::getExtent)
+		.method("getDF", &getVectorAttributes)
+		.method("getGeometryWKT", &SpatVector::getGeometryWKT)
+		.method("isLonLat", &SpatVector::is_lonlat)
+		.method("length", &SpatVector::length)
 //		.field("srs", &SpatVector::srs, "srs")
-		.field("messages", &SpatVector::msg, "messages")
+		.field("messages", &SpatVector::msg)
 		.property("names", &SpatVector::get_names, &SpatVector::set_names)
-		.method("nrow", &SpatVector::nrow, "nrow")
-		.method("ncol", &SpatVector::ncol, "ncol")
-		.method("project", &SpatVector::project, "project")
+		.method("nrow", &SpatVector::nrow)
+		.method("ncol", &SpatVector::ncol)
+		.method("project", &SpatVector::project)
+		.method("project_xy", &SpatVector::project_xy)
 		.method("read", &SpatVector::read)
 		.method("setGeometry", &SpatVector::setGeometry)
 		.method("setPointsXY", &SpatVector::setPointsGeometry)
 		.method("setPointsDF", &SpatVector::setPointsDF)
-		.method("size", &SpatVector::size, "size")
+		.method("size", &SpatVector::size)
 		.method("subset_cols", ( SpatVector (SpatVector::*)(std::vector<int>))( &SpatVector::subset_cols ))
 		.method("subset_rows", ( SpatVector (SpatVector::*)(std::vector<int>))( &SpatVector::subset_rows ))
-		.method("remove_rows", &SpatVector::remove_rows, "remove_rows")
-		.method("type", &SpatVector::type, "type")
+		.method("remove_rows", &SpatVector::remove_rows)
+		.method("type", &SpatVector::type)
 		.method("nullGeoms", &SpatVector::nullGeoms)
 
 		.method("write", &SpatVector::write)
 		.method("delete_layers", &SpatVector::delete_layers)
 		.method("layer_names", &SpatVector::layer_names)
 
-		.method("bienvenue", &SpatVector::bienvenue, "bienvenue")
-		.method("allerretour", &SpatVector::allerretour, "allerretour")
-		.method("geos_isvalid", &SpatVector::geos_isvalid, "geos_isvalid")
-		.method("geos_isvalid_msg", &SpatVector::geos_isvalid_msg, "geos_isvalid_msg")
+//		.method("bienvenue", &SpatVector::bienvenue)
+//		.method("allerretour", &SpatVector::allerretour)
+		.method("geos_isvalid", &SpatVector::geos_isvalid)
+		.method("geos_isvalid_msg", &SpatVector::geos_isvalid_msg)
 
 		.method("aggregate", ( SpatVector (SpatVector::*)(std::string, bool))( &SpatVector::aggregate ))
 		.method("aggregate_nofield", ( SpatVector (SpatVector::*)(bool))( &SpatVector::aggregate ))
@@ -485,7 +509,8 @@ RCPP_MODULE(spat){
 		.method("rotate", &SpatVector::rotate)
 		.method("erase_agg", &SpatVector::erase_agg)
 		.method("erase", ( SpatVector (SpatVector::*)(SpatVector))( &SpatVector::erase ))
-		.method("erase_self", ( SpatVector (SpatVector::*)())( &SpatVector::erase ))
+		.method("erase_self", ( SpatVector (SpatVector::*)(bool))( &SpatVector::erase ))
+		.method("elongate", &SpatVector::elongate)
 		.method("gaps", &SpatVector::gaps)
 
 		.method("symdif", &SpatVector::symdif)
@@ -503,7 +528,7 @@ RCPP_MODULE(spat){
 		.method("mask", &SpatVector::mask)
 
 		.method("is_related", &SpatVector::is_related)
-			
+
 		.method("related_between", ( std::vector<std::vector<double>> (SpatVector::*)(SpatVector, std::string, bool))( &SpatVector::which_relate))
 		.method("related_within", ( std::vector<std::vector<double>> (SpatVector::*)(std::string, bool))( &SpatVector::which_relate))
 
@@ -636,14 +661,16 @@ RCPP_MODULE(spat){
 		.method("setCatIndex", &SpatRaster::setCatIndex)
 		.method("getScaleOffset", &SpatRaster::getScaleOffset)
 		.method("setScaleOffset", &SpatRaster::setScaleOffset)
-	
+
 		.method("hasColors", &SpatRaster::hasColors)
 		.method("getColors", &SpatRaster::getColors)
 		.method("setColors", &SpatRaster::setColors)
 		.method("removeColors", &SpatRaster::removeColors)
 
-		.property("dataType", &SpatRaster::getDataType)
-		.property("valueType", &SpatRaster::getValueType)
+		// need to keep this for raster
+		.property("dataType", &SpatRaster::dataType)
+		.method("getDataType", &SpatRaster::getDataType)
+		.method("valueType", &SpatRaster::getValueType)
 		.method("setValueType", &SpatRaster::setValueType)
 		.property("hasRange", &SpatRaster::hasRange )
 		.property("hasValues", &SpatRaster::hasValues )
@@ -691,20 +718,20 @@ RCPP_MODULE(spat){
 // only if SpatRasterSource is exposed
 //		.field_readonly("source", &SpatRaster::source )
 
-		.method("collapse_sources", &SpatRaster::collapse_sources, "collapse_sources" )
-		.method("make_vrt", &SpatRaster::make_vrt, "make_vrt" )
+		.method("collapse_sources", &SpatRaster::collapse_sources)
+		.method("make_vrt", &SpatRaster::make_vrt)
 
-		.method("dense_extent", &SpatRaster::dense_extent, "dense_extent")
-		.method("setNames", &SpatRaster::setNames, "setNames" )
-		.method("setTime", &SpatRaster::setTime, "setTime" )
-		.method("setDepth", &SpatRaster::setDepth, "setDepth" )
-		.method("setUnit", &SpatRaster::setUnit, "setUnit" )
-		.method("set_resolution", &SpatRaster::setResolution, "set resolution")
-		.method("subset", &SpatRaster::subset, "subset")
+		.method("dense_extent", &SpatRaster::dense_extent)
+		.method("setNames", &SpatRaster::setNames)
+		.method("setTime", &SpatRaster::setTime)
+		.method("setDepth", &SpatRaster::setDepth)
+		.method("setUnit", &SpatRaster::setUnit)
+		.method("set_resolution", &SpatRaster::setResolution)
+		.method("subset", &SpatRaster::subset)
 
 		.method("cellFromXY", ( std::vector<double> (SpatRaster::*)(std::vector<double>,std::vector<double>) )( &SpatRaster::cellFromXY ))
-		.method("vectCells", &SpatRaster::vectCells, "vectCells")
-		.method("extCells", &SpatRaster::extCells, "extCells")
+		.method("vectCells", &SpatRaster::vectCells)
+		.method("extCells", &SpatRaster::extCells)
 
 		.method("wmean_rast", (SpatRaster (SpatRaster::*)(SpatRaster, bool, SpatOptions&))( &SpatRaster::weighted_mean ))
 		.method("wmean_vect", (SpatRaster (SpatRaster::*)(std::vector<double>, bool, SpatOptions&))( &SpatRaster::weighted_mean ))
@@ -717,11 +744,11 @@ RCPP_MODULE(spat){
 		.method("rowFromY", ( std::vector<int_64> (SpatRaster::*)(const std::vector<double>&) )( &SpatRaster::rowFromY ))
 		.method("xyFromCell", ( std::vector< std::vector<double> > (SpatRaster::*)(std::vector<double>&) )( &SpatRaster::xyFromCell ))
 		.method("rowColFromCell", ( std::vector< std::vector<int_64> > (SpatRaster::*)(std::vector<double>) )( &SpatRaster::rowColFromCell ))
-		.method("readStart", &SpatRaster::readStart, "readStart")
-		.method("readStop", &SpatRaster::readStop, "readStop")
-		.method("readAll", &SpatRaster::readAll, "readAll")
-		.method("readValues", &SpatRaster::readValuesR, "readValues")
-		.method("getValues", &SpatRaster::getValues, "getValues")
+		.method("readStart", &SpatRaster::readStart)
+		.method("readStop", &SpatRaster::readStop)
+		.method("readAll", &SpatRaster::readAll)
+		.method("readValues", &SpatRaster::readValuesR)
+		.method("getValues", &SpatRaster::getValues)
 		.method("getBlockSizeR", &getBlockSizeR)
 		.method("getBlockSizeWrite", &getBlockSizeWrite)
 		.method("same", &sameObject)
@@ -730,127 +757,143 @@ RCPP_MODULE(spat){
 #endif
 		.method("setValues", &SpatRaster::setValues)
 		.method("replaceCellValues", &SpatRaster::replaceCellValues)
-		.method("setRange", &SpatRaster::setRange, "setRange")
-		.method("writeStart", &SpatRaster::writeStart, "writeStart")
-		.method("writeStop", &SpatRaster::writeStop, "writeStop")
-		.method("writeValues", &SpatRaster::writeValues, "writeValues")
-		.method("writeRaster", &SpatRaster::writeRaster, "writeRaster")
-		.method("canProcessInMemory", &SpatRaster::canProcessInMemory, "canProcessInMemory")
-		.method("chunkSize", &SpatRaster::chunkSize, "chunkSize")
-		.method("to_memory", &SpatRaster::to_memory, "to_memory")
+		.method("replaceCellValuesLayer", &SpatRaster::replaceCellValuesLayer)
+		.method("setRange", &SpatRaster::setRange)
+		.method("writeStart", &SpatRaster::writeStart)
+		.method("writeStop", &SpatRaster::writeStop)
+		.method("writeValues", &SpatRaster::writeValues)
+		.method("writeRaster", &SpatRaster::writeRaster)
+		.method("canProcessInMemory", &SpatRaster::canProcessInMemory)
+		.method("chunkSize", &SpatRaster::chunkSize)
+//		.method("to_memory", &SpatRaster::to_memory, "to_memory")
+		.method("update_meta", &SpatRaster::update_meta)
 
-		.method("adjacentMat", &SpatRaster::adjacentMat, "adjacent with matrix")
-		.method("adjacent", &SpatRaster::adjacent, "adjacent")
-		.method("aggregate", &SpatRaster::aggregate, "aggregate")
-		.method("align", &SpatRaster::align, "align")
-		.method("apply", &SpatRaster::apply, "apply")
-		.method("rapply", &SpatRaster::rapply, "rapply")
-		.method("rappvals", &SpatRaster::rappvals, "rappvals")
+
+		.method("adjacentMat", &SpatRaster::adjacentMat)
+		.method("adjacent", &SpatRaster::adjacent)
+		.method("aggregate", &SpatRaster::aggregate)
+		.method("align", &SpatRaster::align)
+		.method("apply", &SpatRaster::apply)
+		.method("rapply", &SpatRaster::rapply)
+		.method("rappvals", &SpatRaster::rappvals)
 		.method("arith_rast", ( SpatRaster (SpatRaster::*)(SpatRaster, std::string, SpatOptions&) )( &SpatRaster::arith ))
 		.method("arith_numb", ( SpatRaster (SpatRaster::*)(std::vector<double>, std::string, bool, SpatOptions&) )( &SpatRaster::arith ))
-		.method("rst_area", &SpatRaster::rst_area, "rst_area")
-		.method("sum_area", &SpatRaster::sum_area, "sum_area")
-		.method("area_by_value", &SpatRaster::area_by_value, "area_by_value")
+		.method("rst_area", &SpatRaster::rst_area)
+		.method("sum_area", &SpatRaster::sum_area)
+		.method("area_by_value", &SpatRaster::area_by_value)
 
-		.method("as_points", &SpatRaster::as_points, "as_points")
-		.method("as_lines", &SpatRaster::as_lines, "as_lines")
-		.method("as_polygons", &SpatRaster::as_polygons, "as_polygons")
-		.method("polygonize", &SpatRaster::polygonize, "polygonize")
+		.method("as_points", &SpatRaster::as_points)
+		.method("as_points_value", &SpatRaster::as_points_value)
+		.method("cells_notna", &SpatRaster::cells_notna)
+		.method("as_multipoints", &SpatRaster::as_multipoints)
 
-		.method("atan2", &SpatRaster::atan_2, "atan2")
+		.method("as_lines", &SpatRaster::as_lines)
+		.method("as_polygons", &SpatRaster::as_polygons)
+		.method("polygonize", &SpatRaster::polygonize)
 
-		.method("bilinearValues", &SpatRaster::bilinearValues, "bilin")
+		.method("atan2", &SpatRaster::atan_2)
 
-		.method("patches", &SpatRaster::clumps, "patches")
-		.method("boundaries", &SpatRaster::edges, "edges")
-		.method("buffer", &SpatRaster::buffer, "buffer")
+		.method("bilinearValues", &SpatRaster::bilinearValues)
+
+		.method("patches", &SpatRaster::clumps)
+		.method("boundaries", &SpatRaster::edges)
+		.method("buffer", &SpatRaster::buffer)
 		.method("gridDistance", &SpatRaster::gridDistance)
 		.method("costDistance", &SpatRaster::costDistance)
 		.method("rastDistance", &SpatRaster::distance)
 		.method("rastDirection", &SpatRaster::direction)
 		.method("make_tiles", &SpatRaster::make_tiles)
 		.method("ext_from_rc", &SpatRaster::ext_from_rc)
-	
+
 		.method("combineCats", &SpatRaster::combineCats)
 		.method("droplevels", &SpatRaster::dropLevels)
-
-		.method("vectDisdirRasterize", &SpatRaster::disdir_vector_rasterize)
-		.method("vectDistanceDirect", &SpatRaster::distance_vector)
-		.method("clamp", &SpatRaster::clamp, "clamp")
-		.method("replaceValues", &SpatRaster::replaceValues, "replace")
-		.method("classify", ( SpatRaster (SpatRaster::*)(std::vector<double>, unsigned, unsigned, bool, bool, double, bool, bool, bool, SpatOptions&) )( &SpatRaster::reclassify), "reclassify")
+		.method("vectDistanceDirect", &SpatRaster::distance_spatvector)
+		.method("vectDistanceRasterize", &SpatRaster::distance_rasterize)
+		.method("vectDirectionRasterize", &SpatRaster::direction_rasterize)
+		.method("clamp", &SpatRaster::clamp)
+		.method("replaceValues", &SpatRaster::replaceValues)
+		.method("classify", ( SpatRaster (SpatRaster::*)(std::vector<double>, unsigned, unsigned, bool, bool, double, bool, bool, bool, SpatOptions&) )( &SpatRaster::reclassify))
 		//.method("source_collapse", &SpatRaster::collapse, "collapse")
-		.method("selRange", &SpatRaster::selRange, "selRange")
-		.method("separate", &SpatRaster::separate, "separate")
+		.method("selRange", &SpatRaster::selRange)
+		.method("separate", &SpatRaster::separate)
 		.method("sort", &SpatRaster::sort)
+		.method("intersect", &SpatRaster::intersect)
 
-		.method("cover", &SpatRaster::cover, "cover")
-		.method("crop", &SpatRaster::crop, "crop")
+		.method("cover", &SpatRaster::cover)
+		.method("crop", &SpatRaster::crop)
 		.method("crop_mask", &SpatRaster::cropmask)
-		.method("cum", &SpatRaster::cum, "cum")
-		.method("disaggregate", &SpatRaster::disaggregate, "disaggregate")
-		.method("expand", &SpatRaster::extend, "extend")
-		.method("extractCell", &SpatRaster::extractCell, "extractCell")
-		.method("extractXY", &SpatRaster::extractXY, "extractXY")
+		.method("cum", &SpatRaster::cum)
+		.method("disaggregate", &SpatRaster::disaggregate)
+		.method("expand", &SpatRaster::extend)
+		.method("extractCell", &SpatRaster::extractCell)
+		.method("extractXY", &SpatRaster::extractXY)
 //		.method("extractXYFlat", &SpatRaster::extractXYFlat, "extractXYflat")
-		.method("extractVector", &SpatRaster::extractVector, "extractVector")
-		.method("extractVectorFlat", &SpatRaster::extractVectorFlat, "extractVectorFlat")
-		.method("flip", &SpatRaster::flip, "flip")
-		.method("focal", &SpatRaster::focal, "focal")
-		.method("focalValues", &SpatRaster::focal_values, "focalValues")
-		.method("count", &SpatRaster::count, "count")
-		.method("freq", &SpatRaster::freq, "freq")
-		.method("geometry", &SpatRaster::geometry, "geometry")
+		.method("extractVector", &SpatRaster::extractVector)
+		.method("extractVectorFlat", &SpatRaster::extractVectorFlat)
+		.method("flip", &SpatRaster::flip)
+		.method("focal", &SpatRaster::focal)
+		.method("focalValues", &SpatRaster::focal_values)
+		.method("count", &SpatRaster::count)
+		.method("freq", &SpatRaster::freq)
+		.method("geometry", &SpatRaster::geometry)
 
-		.method("get_aggregates", &SpatRaster::get_aggregates, "get_aggregates")
-		.method("get_aggregate_dims", &SpatRaster::get_aggregate_dims2, "get_aggregate_dims")
-		.method("global", &SpatRaster::global, "global")
-		.method("global_weighted_mean", &SpatRaster::global_weighted_mean, "global weighted mean")
+		.method("get_aggregates", &SpatRaster::get_aggregates)
+		.method("get_aggregate_dims", &SpatRaster::get_aggregate_dims2)
+		.method("global", &SpatRaster::global)
+		.method("global_weighted_mean", &SpatRaster::global_weighted_mean)
 
 		.method("initf", ( SpatRaster (SpatRaster::*)(std::string, bool, SpatOptions&) )( &SpatRaster::init ), "init fun")
 		.method("initv", ( SpatRaster (SpatRaster::*)(std::vector<double>, SpatOptions&) )( &SpatRaster::init ), "init value")
-		.method("is_in", &SpatRaster::is_in, "isin")
-		.method("is_in_cells", &SpatRaster::is_in_cells, "isincells")
+		.method("is_in", &SpatRaster::is_in)
+		.method("is_in_cells", &SpatRaster::is_in_cells)
+		.method("anynan", &SpatRaster::anynan)
+		.method("allnan", &SpatRaster::allnan)
 		.method("isnan", &SpatRaster::isnan)
 		.method("not_na", &SpatRaster::isnotnan)
-		.method("isfinite", &SpatRaster::isfinite, "isfinite")
-		.method("isinfinite", &SpatRaster::isinfinite, "isinfinite")
+		.method("isfinite", &SpatRaster::isfinite)
+		.method("isinfinite", &SpatRaster::isinfinite)
 		.method("logic_rast", ( SpatRaster (SpatRaster::*)(SpatRaster, std::string, SpatOptions&) )( &SpatRaster::logic ))
 		.method("logic_numb", ( SpatRaster (SpatRaster::*)(bool, std::string, SpatOptions&) )( &SpatRaster::logic ))
-		.method("mask_raster", ( SpatRaster (SpatRaster::*)(SpatRaster, bool, std::vector<double>, double, SpatOptions&) )( &SpatRaster::mask), "mask raster")
-		.method("mask_vector", ( SpatRaster (SpatRaster::*)(SpatVector, bool, double, bool, SpatOptions&) )( &SpatRaster::mask), "mask vector")
-		.method("math", &SpatRaster::math, "math")
-		.method("math2", &SpatRaster::math2, "math2")
-		.method("modal", &SpatRaster::modal, "modal")
-		.method("quantile", &SpatRaster::quantile, "quantile")
+		.method("mask_raster", ( SpatRaster (SpatRaster::*)(SpatRaster, bool, std::vector<double>, double, SpatOptions&) )( &SpatRaster::mask))
+		.method("mask_vector", ( SpatRaster (SpatRaster::*)(SpatVector, bool, double, bool, SpatOptions&) )( &SpatRaster::mask))
+		.method("math", &SpatRaster::math)
+		.method("math2", &SpatRaster::math2)
+		.method("modal", &SpatRaster::modal)
+		.method("quantile", &SpatRaster::quantile)
 		.method("rasterize", &SpatRaster::rasterize)
 		.method("rasterizePoints", &SpatRaster::rasterizePoints)
 		.method("rasterizeLyr", &SpatRaster::rasterizeLyr)
 		.method("rasterizeGeom", &SpatRaster::rasterizeGeom)
+		.method("rasterizeWindow", &SpatRaster::rasterizeWindow)
+		.method("wincircle", &SpatRaster::win_circle)
+		.method("winrect", &SpatRaster::win_rect)
 		.method("rgb2col", &SpatRaster::rgb2col)
 		.method("rgb2hsx", &SpatRaster::rgb2hsx)
 		.method("hsx2rgb", &SpatRaster::hsx2rgb)
 
-		.method("reverse", &SpatRaster::reverse, "reverse")
-		.method("rotate", &SpatRaster::rotate, "rotate")
+		.method("reverse", &SpatRaster::reverse)
+		.method("rotate", &SpatRaster::rotate)
 		//.method("sampleCells", &SpatRaster::sampleCells, "sampleCells")
-		.method("sampleRegularRaster", &SpatRaster::sampleRegularRaster, "sampleRegular")
-		.method("sampleRowColRaster", &SpatRaster::sampleRowColRaster, "sampleRowCol")
-		.method("sampleRegularValues", &SpatRaster::sampleRegularValues, "sampleValues")
-		.method("sampleRowColValues", &SpatRaster::sampleRowColValues, "sampleRowCol")
-		.method("sampleRandomRaster", &SpatRaster::sampleRandomRaster, "sampleRandom")
-		.method("sampleRandomValues", &SpatRaster::sampleRandomValues, "sampleValues")
-		.method("scale", &SpatRaster::scale, "scale")
-		.method("shift", &SpatRaster::shift, "shift")
-		.method("terrain", &SpatRaster::terrain, "terrain")
-		.method("summary", &SpatRaster::summary, "summary")
-		.method("summary_numb", &SpatRaster::summary_numb, "summary_numb")
-		.method("transpose", &SpatRaster::transpose, "transpose")
-		.method("trig", &SpatRaster::trig, "trig")
-		.method("trim", &SpatRaster::trim)
+		.method("sampleRegularRaster", &SpatRaster::sampleRegularRaster)
+		.method("sampleRowColRaster", &SpatRaster::sampleRowColRaster)
+		.method("sampleRegularValues", &SpatRaster::sampleRegularValues)
+		.method("sampleRowColValues", &SpatRaster::sampleRowColValues)
+		.method("sampleRandomRaster", &SpatRaster::sampleRandomRaster)
+		.method("sampleRandomValues", &SpatRaster::sampleRandomValues)
+		.method("scale", &SpatRaster::scale)
+		.method("shift", &SpatRaster::shift)
+		.method("terrain", &SpatRaster::terrain)
+		.method("summary", &SpatRaster::summary)
+		.method("summary_numb", &SpatRaster::summary_numb)
+		.method("transpose", &SpatRaster::transpose)
+		.method("trig", &SpatRaster::trig)
+		.method("trim1", &SpatRaster::trim1)
+		.method("trim", &SpatRaster::trim2)
 		.method("unique", &SpatRaster::unique)
 		.method("where", &SpatRaster::where)
-		.method("sieve", &SpatRaster::sieveFilter, "sievefilter")
+		.method("sieve", &SpatRaster::sieveFilter)
+		.method("view", &SpatRaster::viewshed)
+		.method("proximity", &SpatRaster::proximity)
 
 		.method("rectify", &SpatRaster::rectify)
 		.method("stretch", &SpatRaster::stretch)
@@ -864,7 +907,13 @@ RCPP_MODULE(spat){
 
     class_<SpatRasterCollection>("SpatRasterCollection")
 		.constructor()
-		.method("deepcopy", &SpatRasterCollection::deepCopy, "deepCopy")
+	    .constructor<std::string, std::vector<int>, bool>()
+
+		.property("names", &SpatRasterCollection::get_names, &SpatRasterCollection::set_names)
+
+		.method("deepcopy", &SpatRasterCollection::deepCopy)
+		.method("dims", &SpatRasterCollection::dims)
+		.method("extent", &SpatRasterCollection::getExtent)
 
 		.method("has_error", &SpatRasterCollection::has_error)
 		.method("has_warning", &SpatRasterCollection::has_warning)
@@ -872,13 +921,13 @@ RCPP_MODULE(spat){
 		.method("getWarnings", &SpatRasterCollection::getWarnings)
 		//.field("messages", &SpatRasterCollection::msg, "messages")
 		.field_readonly("x", &SpatRasterCollection::ds)
-		.method("length", &SpatRasterCollection::size, "size")
-		.method("resize", &SpatRasterCollection::resize, "resize")
-		.method("erase", &SpatRasterCollection::erase, "erase")
-		.method("add", &SpatRasterCollection::push_back, "push_back")
-		.method("merge", &SpatRasterCollection::merge, "merge")
-		.method("mosaic", &SpatRasterCollection::mosaic, "mosaic")
-		.method("morph", &SpatRasterCollection::morph, "morph")
+		.method("length", &SpatRasterCollection::size)
+		.method("resize", &SpatRasterCollection::resize)
+		.method("erase", &SpatRasterCollection::erase)
+		.method("add", &SpatRasterCollection::push_back)
+		.method("merge", &SpatRasterCollection::merge)
+		.method("mosaic", &SpatRasterCollection::mosaic)
+		.method("morph", &SpatRasterCollection::morph)
 	;
 
     class_<SpatRasterStack>("SpatRasterStack")
@@ -892,26 +941,31 @@ RCPP_MODULE(spat){
 		.method("getError", &SpatRasterStack::getError)
 		.method("getWarnings", &SpatRasterStack::getWarnings)
 
-		.method("readStart", &SpatRasterStack::readStart, "readStart")
-		.method("readStop", &SpatRasterStack::readStop, "readStop")
-		.method("nsds", &SpatRasterStack::nsds, "")
-		.method("ncol", &SpatRasterStack::ncol, "")
-		.method("nrow", &SpatRasterStack::nrow, "")
-		.method("getSRS", &SpatRasterStack::getSRS, "")
+		.method("readStart", &SpatRasterStack::readStart)
+		.method("readStop", &SpatRasterStack::readStop)
+		.method("nsds", &SpatRasterStack::nsds)
+		.method("ncol", &SpatRasterStack::ncol)
+		.method("nrow", &SpatRasterStack::nrow)
+		.method("nlyr", &SpatRasterStack::nlyr)
+		.method("res", &SpatRasterStack::resolution)
+		.method("ext", &SpatRasterStack::getExtent)
+		.method("filenames", &SpatRasterStack::filenames)
+		
+		.method("get_crs", &SpatRasterStack::getSRS)
 		.property("names", &SpatRasterStack::get_names, &SpatRasterStack::set_names)
 		.property("long_names", &SpatRasterStack::get_longnames, &SpatRasterStack::set_longnames)
 		.property("units", &SpatRasterStack::get_units, &SpatRasterStack::set_units)
-		.method("add", &SpatRasterStack::push_back, "")
-		.method("resize", &SpatRasterStack::resize, "")
-		.method("summary", &SpatRasterStack::summary, "summary")
-		.method("summary_numb", &SpatRasterStack::summary_numb, "summary_numb")
-		.method("getsds", &SpatRasterStack::getsds, "")
-		.method("replace", &SpatRasterStack::replace, "")
-		.method("subset", &SpatRasterStack::subset, "")
-		.method("collapse", &SpatRasterStack::collapse , "")
-		.method("extractCell", &SpatRasterStack::extractCell, "extractCell")
-		.method("extractVector", &SpatRasterStack::extractVector, "extractVector")
-		.method("crop", &SpatRasterStack::crop, "")
+		.method("add", &SpatRasterStack::push_back)
+		.method("resize", &SpatRasterStack::resize)
+		.method("summary", &SpatRasterStack::summary)
+		.method("summary_numb", &SpatRasterStack::summary_numb)
+		.method("getsds", &SpatRasterStack::getsds)
+		.method("replace", &SpatRasterStack::replace)
+		.method("subset", &SpatRasterStack::subset)
+		.method("collapse", &SpatRasterStack::collapse )
+		.method("extractCell", &SpatRasterStack::extractCell)
+		.method("extractVector", &SpatRasterStack::extractVector)
+		.method("crop", &SpatRasterStack::crop)
 	;
 }
 
