@@ -725,12 +725,15 @@ std::vector<std::vector<std::vector<double>>> SpatRaster::extractVector(SpatVect
 			if (weights) {
 				if (gtype == "lines") {
 					rasterizeLinesLength(cell, wgt, p, opt);
+				} else {
+					rasterizeCellsWeights(cell, wgt, p, opt);
 				}
 			} else if (exact) {
-				if (gtype == "lines") {	
+				if (gtype == "lines") {
 					rasterizeLinesLength(cell, wgt, p, opt);
+				} else {
+					rasterizeCellsExact(cell, wgt, p, opt);
 				}
-				rasterizeCellsExact(cell, wgt, p, opt);
 			} else {
 				cell = rasterizeCells(p, touches, opt);
             }
@@ -764,7 +767,7 @@ std::vector<double> SpatRaster::extractVectorFlat(SpatVector v, bool touches, st
 		exact = false;
 	}
 	if (exact) weights = false;
-	
+
     unsigned nl = nlyr();
     unsigned ng = v.size();
 
@@ -993,7 +996,7 @@ std::vector<double> SpatRaster::extractCellFlat(std::vector<double> &cell) {
 	std::vector<double> out(nlyr() * n, NAN);
 
 	unsigned ns = nsrc();
-	unsigned lyr = 0;
+//	unsigned lyr = 0;
 	size_t nc;
 	size_t off = 0;
 	for (size_t src=0; src<ns; src++) {
@@ -1033,7 +1036,7 @@ std::vector<double> SpatRaster::extractCellFlat(std::vector<double> &cell) {
 						}
 					}
 				}
-				lyr++;
+				//lyr++;
 			}
 		} else {
 			//if (source[0].driver == "raster") {
