@@ -183,7 +183,7 @@ SpatCategories GetRAT(GDALRasterAttributeTable *pRAT) {
 		return(out);
 	}
 	id.insert(id.end(), id2.begin(), id2.end());
-	
+
 
 	if (!hasvalue) {
 		std::vector<long> vid(nr);
@@ -545,6 +545,7 @@ inline std::string dtypename(const std::string &d) {
 	if (d == "Float32") return "FLT4S";
 	if (d == "Int32") return "INT4S";
 	if (d == "Int16") return "INT2S";
+	if (d == "Int8") return "INT1S";
 	if (d == "UInt32") return "INT4U";
 	if (d == "UInt16") return "INT2U";
 	if (d == "Byte") return "INT1U";
@@ -552,6 +553,7 @@ inline std::string dtypename(const std::string &d) {
 }
 
 
+
 SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool useids) {
 
 	std::vector<std::string> ops;
@@ -583,7 +585,6 @@ SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool u
 		ids.resize(meta.size());
 		std::iota(ids.begin(), ids.end(), 0);
 	}
-	SpatRaster sub;
 	int idssz = ids.size();
 	int metsz = meta.size();
 
@@ -598,6 +599,7 @@ SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool u
 			size_t pos = s.find(delim);
 			if (pos != std::string::npos) {
 				s.erase(0, pos + delim.length());
+				SpatRaster sub;
 				if (sub.constructFromFile(s, {-1}, {""}, {}, {})) {
 					if (!push_back(sub, basename_sds(s), "", "", true)) {
 						addWarning("skipped (different geometry): " + s);
@@ -612,6 +614,64 @@ SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool u
 }
 
 
+SpatRasterCollection::SpatRasterCollection(std::string fname, std::vector<int> ids, bool useids) {
+
+	std::vector<std::string> ops;
+    GDALDataset *poDataset = openGDAL(fname, GDAL_OF_RASTER | GDAL_OF_READONLY | GDAL_OF_VERBOSE_ERROR, ops, ops);
+    if( poDataset == NULL )  {
+		if (!file_exists(fname)) {
+			setError("file does not exist");
+		} else {
+			setError("cannot read from " + fname );
+		}
+		return;
+	}
+
+	std::string delim = "NAME=";
+	char **metadata = poDataset->GetMetadata("SUBDATASETS");
+
+	if (metadata == NULL) {
+		setError("file has no subdatasets");
+		GDALClose( (GDALDatasetH) poDataset );
+		return;
+	}
+
+	std::vector<std::string> meta;
+    for (size_t i=0; metadata[i] != NULL; i++) {
+		meta.push_back(metadata[i]);
+	}
+
+	if (!useids) {
+		ids.resize(meta.size());
+		std::iota(ids.begin(), ids.end(), 0);
+	}
+	int idssz = ids.size();
+	int metsz = meta.size();
+
+	if (metsz == 0) {
+		setError("file does not consist of subdatasets");
+	} else {
+		for (int i=0; i<idssz; i++) {
+			if ((ids[i] < 0) || ((2*ids[i]) >= metsz)) {
+				continue;
+			}
+			std::string s = meta[ids[i]*2];
+			size_t pos = s.find(delim);
+			if (pos != std::string::npos) {
+				s.erase(0, pos + delim.length());
+				SpatRaster sub;
+				if (sub.constructFromFile(s, {-1}, {""}, {}, {})) {
+					push_back(sub, basename_sds(s));
+				} else {
+					addWarning("skipped (fail): " + s);
+				}
+			}
+		}
+	}
+	GDALClose( (GDALDatasetH) poDataset );
+}
+
+
 /*
 SpatRaster SpatRaster::fromFiles(std::vector<std::string> fname, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> drivers, std::vector<std::string> options) {
 	SpatRaster out;
@@ -823,7 +883,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		poBand->GetBlockSize(&bs1, &bs2);
 		s.blockcols[i] = bs1;
 		s.blockrows[i] = bs2;
-		s.dataType[i] = dtypename(GDALGetDataTypeName(poBand->GetRasterDataType()));
+		s.dtype = dtypename(GDALGetDataTypeName(poBand->GetRasterDataType()));
 
 		adfMinMax[0] = poBand->GetMinimum( &bGotMin );
 		adfMinMax[1] = poBand->GetMaximum( &bGotMax );
@@ -931,8 +991,10 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	if ((gdrv=="netCDF") || (gdrv == "HDF5"))  {
 		std::vector<std::string> metadata;
 		char **m = poDataset->GetMetadata();
-		while (*m != nullptr) {
-			metadata.push_back(*m++);
+		if (m) {
+			while (*m != nullptr) {
+				metadata.push_back(*m++);
+			}
 		}
 		s.set_names_time_ncdf(metadata, bandmeta, msg);
 	} else if (gdrv == "GRIB") {
@@ -1687,6 +1749,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 		if (fc & fu) break;
 	}
 
+	bool years = false;
 	bool days = false;
 	bool hours = false;
 	bool seconds = false;
@@ -1699,6 +1762,8 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 			days = true;
 		} else if ((origin.find("seconds")) != std::string::npos) {
 			seconds = true;
+		} else if ((origin.find("years")) != std::string::npos) {
+			years = true;
 		}
 		size_t pos;
 		if ((pos = origin.find("from")) != std::string::npos) {
@@ -1735,6 +1800,10 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 		} else if (seconds) {
 			offset = get_time_string(origin);
 			for (size_t i=0; i<raw.size(); i++) out.push_back(raw[i]+offset);
+		} else if (years) {
+			step = "years";
+			int syear = getyear(origin);
+			for (size_t i=0; i<raw.size(); i++) out.push_back(get_time(syear+raw[i], 6, 30, 0, 0, 0));
 		} else {
 			step = "raw";
 			for (size_t i=0; i<raw.size(); i++) out.push_back(raw[i]);
@@ -1838,12 +1907,18 @@ void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, st
 
 std::vector<std::vector<std::string>> grib_names(const std::vector<std::vector<std::string>> &m) {
 
-	std::vector<std::vector<std::string>> out(3);
+	std::vector<std::vector<std::string>> out(4);
 	if (m.size() < 1) return out;
+	
+	bool ft1 = false;
+	bool ft2 = false;
 
 	for (size_t i=0; i<m.size(); i++) {
-		std::string comm, time, units = "";
+
+		std::string comm, time1, time2, units = "";
+
 		for (size_t j=0; j<m[i].size(); j++) {
+			
 			size_t pos = m[i][j].find("GRIB_COMMENT=");
 			if (pos != std::string::npos) {
 				comm = m[i][j];
@@ -1860,6 +1935,18 @@ std::vector<std::vector<std::string>> grib_names(const std::vector<std::vector<s
 				lrtrim(units);
 				continue;
 			}
+			pos = m[i][j].find("GRIB_VALID_TIME=");
+			if (pos != std::string::npos) {
+				std::string tmp = m[i][j];
+				tmp.erase(0, pos+16);
+				pos = tmp.find("sec");
+				if (pos != std::string::npos) {
+					tmp.erase(tmp.begin()+pos, tmp.end());
+				}
+				time1 = tmp;
+				ft1 = true;
+				continue;
+			}
 			pos = m[i][j].find("TIME=");
 			if (pos != std::string::npos) {
 				std::string tmp = m[i][j];
@@ -1867,15 +1954,23 @@ std::vector<std::vector<std::string>> grib_names(const std::vector<std::vector<s
 				pos = tmp.find("sec");
 				if (pos != std::string::npos) {
 					tmp.erase(tmp.begin()+pos, tmp.end());
-					time = tmp;
 				}
-				continue;
+				time2 = tmp;
+				ft2 = true;
 			}
 		}
 		out[0].push_back(comm);
 		out[1].push_back(units);
-		out[2].push_back(time);
+		out[2].push_back(time1);
+		out[3].push_back(time2);
 	}
+	
+	if (!ft1) {
+		if (ft2) {
+			out[2] = out[3];
+		}
+	}
+	out.resize(3);
 	return out;
 }
 
@@ -1884,6 +1979,7 @@ std::vector<std::vector<std::string>> grib_names(const std::vector<std::vector<s
 void SpatRasterSource::set_names_time_grib(std::vector<std::vector<std::string>> bandmeta, std::string &msg) {
 
 	if (bandmeta.size() == 0) return;
+	
 	std::vector<std::vector<std::string>> nms = grib_names(bandmeta);
 
 	if (nms[0].size() != names.size()) return;
@@ -1893,28 +1989,36 @@ void SpatRasterSource::set_names_time_grib(std::vector<std::vector<std::string>>
 		str_replace(names[i], "0[-] ", "");
 		str_replace_all(names[i], "\"", "");
 	}
-	unit = {nms[1]};
 
-	std::vector<int_64> tm;
-	bool hastime = true;
-	for (size_t i=0; i<nms[2].size(); i++) {
-		if (nms[2][i] == "") {
-			hastime = false;
-			break;
-		}
-		int_64 tim;
-		try {
-			tim = stol(nms[2][i]);
-		} catch(...) {
-			hastime = false;
-			break;
-		}
-		tm.push_back(tim);
+	if (nms[1].size() == nms[0].size()) {
+		unit = {nms[1]};
 	}
+
+	bool hastime = false;
+	std::vector<int_64> tm;
+	if (nms[2].size() == nms[0].size()) {
+		hastime = true;
+		int_64 tim;
+		for (size_t i=0; i<nms[2].size(); i++) {
+			if (nms[2][i] == "") {
+				hastime = false;
+				break;
+			}
+			try {
+				tim = stol(nms[2][i]);
+			} catch(...) {
+				hastime = false;
+				break;
+			}
+			tm.push_back(tim);
+		}
+	}
+
 	if (hastime) {
 		time = tm;
 		timestep = "seconds";
 		hasTime = true;
 	}
+	
 }
 
