@@ -377,6 +377,9 @@ SpatGeom emptyGeom() {
 
 bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what) {
 
+//Rcpp::Rcout << 1 << std::endl;
+//R_FlushConsole(); R_ProcessEvents();
+
 	std::string crs = "";
 
 	OGRLayer *poLayer;
@@ -429,10 +432,18 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 		}
 	}
 
+
 	OGRSpatialReference *poSRS = poDS->GetLayer(0)->GetSpatialRef();
+
 	if (poSRS) {
 		char *psz = NULL;
+	#if GDAL_VERSION_MAJOR >= 3
+		const char *options[3] = { "MULTILINE=YES", "FORMAT=WKT2", NULL };
+		OGRErr err = poSRS->exportToWkt(&psz, options);
+	#else
 		OGRErr err = poSRS->exportToWkt(&psz);
+	#endif
+
 		if (err == OGRERR_NONE) {
 			crs = psz;
 		}
@@ -474,10 +485,10 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 	if (what == "attributes") {
 		if (query != "") {
 			poDS->ReleaseResultSet(poLayer);
-		}		
+		}
 		return true;
 	}
-	
+
 	//const char* lname = poLayer->GetName();
 	OGRwkbGeometryType wkbgeom = wkbFlatten(poLayer->GetGeomType());
 	OGRFeature *poFeature;
