@@ -15,6 +15,9 @@
 // You should have received a copy of the GNU General Public License
 // along with spat. If not, see <http://www.gnu.org/licenses/>.
 
+#ifndef SPATBASE_GUARD
+#define SPATBASE_GUARD
+
 #include <vector>
 #include <algorithm>
 #include <string>
@@ -71,11 +74,8 @@ class SpatMessages {
 			warnings.push_back(s);
 		}
 
-		std::string getWarnings() {
-			std::string w = "";
-			for (size_t i = 0; i<warnings.size(); i++) {
-				w += warnings[i] + "\n" ;
-			}
+		std::vector<std::string> getWarnings() {
+			std::vector<std::string> w = warnings; 		
 			warnings.resize(0);
 			has_warning = false;
 			return w;
@@ -90,13 +90,14 @@ class SpatMessages {
 			message = s;
 		}
 		
-		std::vector<std::string> getAll() {
+/*		std::vector<std::string> getAll() {
 			std::string warns = getWarnings();
 			std::string error = getError();
 			std::string msg = getMessage();
 			std::vector<std::string> amsgs = { error, warns, msg};
 			return amsgs;
 		}
+*/
 };
 
 
@@ -266,9 +267,12 @@ class SpatExtent {
 		bool valid() {
 			return ((xmax >= xmin) && (ymax >= ymin));
 		}
-		bool valid_notequal() {
+		bool valid_notempty() {
 			return ((xmax > xmin) && (ymax > ymin));
 		}
+		bool empty() {
+			return ((xmax <= xmin) || (ymax <= ymin));
+		}
 
 		bool compare(SpatExtent e, std::string oper, double tolerance);
 
@@ -348,3 +352,18 @@ class SpatSRS {
 		}
 };
 
+
+class SpatProgress {
+	public:
+		virtual ~SpatProgress(){}		
+		size_t nstep;
+		size_t step;
+		std::vector<int> steps;
+		void init(size_t n, int nmin);
+		bool show = false;
+		void stepit();
+		void interrupt();
+};
+
+
+#endif
