@@ -377,7 +377,7 @@ void SpatVector::reserve(size_t n) {
 void SpatVector::computeExtent() {
 	if (geoms.size() == 0) return;
 	extent = geoms[0].extent;
-	for (size_t i=1; i<geoms.size(); i++) {
+	for (size_t i=1; i<geoms.size(); i++) {	
 		extent.unite(geoms[i].extent);
 	}
 }
@@ -398,7 +398,6 @@ bool SpatVector::replaceGeom(SpatGeom p, unsigned i) {
 	if (i < geoms.size()) {
 		if ((geoms[i].extent.xmin == extent.xmin) || (geoms[i].extent.xmax == extent.xmax) ||
 			(geoms[i].extent.ymin == extent.ymin) || (geoms[i].extent.ymax == extent.ymax)) {
-
 			geoms[i] = p;
 			computeExtent();
 		} else {
@@ -1045,7 +1044,7 @@ SpatVector SpatVector::as_points(bool multi, bool skiplast) {
 	if (multi) {
 		v.df = df;
 	} else {
-		v = v.disaggregate();
+		v = v.disaggregate(false);
 	}
 	return(v);
 }
@@ -1227,12 +1226,14 @@ SpatVector SpatVector::round(int digits) {
 				}
 			}
 		}
+		out.geoms[i].computeExtent();
 	}
+	out.computeExtent();
 	return(out);
 }
 
 
-SpatVector SpatVector::normalize_dateline() {
+SpatVector SpatVector::normalize_longitude() {
 	SpatVector out = *this;
 	SpatExtent e = {180, 361, -91, 91};
 	SpatVector x = out.crop(e);
@@ -1254,6 +1255,47 @@ SpatVector SpatVector::normalize_dateline() {
 }
 
 
+SpatVector SpatVector::rotate_longitude(double longitude, bool left) {
+	SpatVector out = *this;
+	size_t ng = out.size();
+	for (size_t i=0; i<ng; i++) {
+		size_t np = out.geoms[i].size();
+		for (size_t j=0; j<np; j++) {
+			size_t nx = out.geoms[i].parts[j].x.size();
+			for (size_t k=0; k<nx; k++) {
+				if (left) {
+					if (out.geoms[i].parts[j].x[k] > longitude) {
+						out.geoms[i].parts[j].x[k] = out.geoms[i].parts[j].x[k] - 360;
+					}
+				} else {
+					if (out.geoms[i].parts[j].x[k] < longitude) {
+						out.geoms[i].parts[j].x[k] = out.geoms[i].parts[j].x[k] + 360;
+					}
+				}
+			}
+			if (out.geoms[i].parts[j].hasHoles()) {
+				size_t nh = out.geoms[i].parts[j].holes.size();
+				for (size_t k=0; k<nh; k++) {
+					size_t nx = out.geoms[i].parts[j].holes[k].x.size();
+					for (size_t h=0; h<nx; h++) {
+						if (left) {
+							if (out.geoms[i].parts[j].holes[k].x[h] > longitude) {
+								out.geoms[i].parts[j].holes[k].x[h] = out.geoms[i].parts[j].holes[k].x[h] - 360;
+							} 
+						} else {
+							if (out.geoms[i].parts[j].holes[k].x[h] < longitude) {
+								out.geoms[i].parts[j].holes[k].x[h] = out.geoms[i].parts[j].holes[k].x[h] + 360;
+							} 
+						}
+					}
+				}
+			}
+		}
+		out.geoms[i].computeExtent();
+	}
+	out.computeExtent();
+	return(out);
+}
 
 
 std::vector<std::vector<std::vector<double>>> SpatVector::linesList() {
@@ -1334,7 +1376,7 @@ std::vector<std::vector<std::vector<std::vector<double>>>> SpatVector::polygonsL
 					out[i][j][0].push_back(NAN);
 					out[i][j][1].push_back(NAN);
 					out[i][j][0].insert(out[i][j][0].end(), g.parts[j].holes[k].x.begin(), g.parts[j].holes[k].x.end());
-					out[i][j][1].insert(out[i][j][1].end(), g.parts[j].holes[k].y.begin(), g.parts[j].holes[k].y.end());				
+					out[i][j][1].insert(out[i][j][1].end(), g.parts[j].holes[k].y.begin(), g.parts[j].holes[k].y.end());
 				}
 			} else {
 				out[i][j][0] = g.parts[j].x;
